TITLE	TRIDENT.ASM

COMMENT
	Most of the info came from Ferraro's book - "Programmer's Guide
	to Super VGA", Second Edition.
	The chip test was lifted (slightly modified) from StoneSoup Group's
	Fractint for DOS 17.1 (VIDEO.ASM). There was a typo in the Ferraro.

	Of the switches shown, only the first four are actual hardware
	switches (which is which, is a mystery to me).
	sw4 is	bus version 	-	off==AT 	on==PS/2
	sw5 is	I/O addressing	-	off==2xx	on==3xx
	sw6 is	ROM data width	-	off==8 bit	on==16bit
	sw7 is	ROM enable/disable.

	The program must be assembled with MASM 6.0. The code is free, but
	if you make any cute changes, or find bugs, please let me know.

	Anton Sachs
	BIX:	garun
	CIS:	70162,1562

	P.S. - original version was for .486, some of the string manipulation
	got a bit ugly in conversion to .86


.model TINY
include equates.inc				; basic stuff - NULL, NL, etc..

clear	MACRO	register
		xor register,register
ENDM

@tell	MACRO	string
		mov si,offset string
		call outString
ENDM


.code
			ORG 80h
comlineSize BYTE	?
comline 	STRING	127 dup (?)
.startup
		jmp start			; bypass the data

.data
Copyright	STRING	'Copywright Anton Sachs, 1992',0
FormatMsg	STRING	NL,"Syntax:  TRIDBUS [8|16]",Zend
TridMsg		STRING	NL,"The card is Trident, chip version 8"
chip		STRING	"8"
			STRING	"00.",NL,"The memory size is: "
vMemory		DWORD	"256K"
			STRING	Zend
isNotMsg	STRING	NL,"The card is NOT Trident.",Zend
AttemptMsg	STRING	NL,"Attempting to reconfigure to ",NUL
NowMsg		STRING	NL,NL,"Now the "
BusMsg		STRING	"card is configured for "
bWidth		WORD	"61"
			STRING	" bit bus.",Zend
PromptMsg	STRING	NL,"Now press any key...",Zend
SwitchMsg	STRING	NL,"Switches are:",NL
			STRING	" 0 1 2 3 4 5 6 7",NL
			STRING	" "
sw0 		BYTE	""
			BYTE	" "
sw1 		BYTE	""
			BYTE	" "
sw2 		BYTE	""
			BYTE	" "
sw3 		BYTE	""
			BYTE	" "
sw4 		BYTE	""
			BYTE	" "
sw5 		BYTE	""
			BYTE	" "
sw6 		BYTE	""
			BYTE	" "
sw7 		BYTE	""
			STRING	" ",Zend


PUreg2		BYTE	?
m1M 		STRING	"1Meg"
m512		STRING	"512K"
m256		STRING	"256K"

.code

outString	PROC
		   .WHILE  1
			  lodsb 						; Get character ASCIIZ string
		   .BREAK .IF al == 0				; Break if null terminator
			  int	  29h					; Display text, advance cursor
		   .ENDW
			ret

outString	ENDP


IsItTrident PROC
		mov	dx,3c4h 		;Test for Trident
		mov	ax,000bh
		out	dx,al
		inc	dl
		xchg ah,al			; dummy write
		out dx,al
		in	al,dx
		cmp	al,06h
		ja	notTrident
		cmp	al,2
		jb	notTrident
		cmp	al,3
		jb	no89
		mov chip,"9"
		mov	dx,3d5h
		mov	al,1fh
		out	dx,al
		inc	dx
		in	al,dx
		and	al,3
		cmp	al,1
		jb	notmem
		mov	si,offset m512
		je	notmem
		mov	si,offset m1M
Texit:	clc 					; is trident
exit:
		ret

notmem:
		mov si,offset m256
		jmp	Texit				; 256K

no89:
		mov	si,offset m512
		mov chip,"8"
		jmp Texit

notTrident:
		stc 					; carry
		jmp exit

IsItTrident ENDP

GetSwitches PROC
		mov dx,03C4h
		mov ax,000Fh
		out dx,ax				; Power #2

		inc dx
		in al,dx

		mov bx,offset sw1
		mov di,0				; eight bits to do

nextSwitch:
		shr al,1
		jnc testBX
		mov bpt [bx][di],""
testBX:
		add di,2
		cmp di,0Eh				; number of switches * 2
		jbe nextSwitch

tellSwitches:
		@tell SwitchMsg
		ret
GetSwitches ENDP

GetBus	PROC
		mov dx,03C4h
		mov ax,000Bh
		out dx,ax				; select Old mode

		mov dx,03C4h
		mov ax,000Eh			; Control reg #1
		out dx,al
		inc dl
		in al,dx
		mov PUreg2,al			; save it for setting
		and al,8				; Bus width size ?
		cmp al,8
		je is16
		mov bWidth,"8 "
		jmp exit
is16:
		mov bWidth,"61"
exit:
		ret

GetBus	ENDP

SetBus	PROC
; value passed in BX

		mov dx,03C4h
		mov ax,000Bh			; set up Old Mode control reg
		out dx,ax

		mov dx,03C4h
		mov ax,000Eh			; Control reg #1
		out dx,ax				; select it

		mov al,PUreg2			; old value
		and al,NOT 8			; clear the bus value
		or al,bl				; set it to user value
		inc dl
		out dx,al
		ret

SetBus	ENDP

main  PROC						; fake procedure

tellFormat:
		@tell FormatMsg
		jmp OKexit

itIsNot:
		@tell isNotMsg
		stc
		mov al,1
		jmp exit

start::
		call IsItTrident
		jc itIsNot
		mov di,offset vMemory
		movsw
		movsw
		@tell TridMsg
		call GetSwitches

		call GetBus
		@tell BusMsg
		cmp comlineSize,NUL
		je tellFormat
		mov di,81h
		clear cx
		mov cl, bpt comlineSize
		mov al,SPACE
		repe scasb
		dec di					; scroll back
		mov si,di
		lodsw					; get the first letter
		mov bWidth,ax
		cmp al,"8"
		je setTo8
		cmp ax,"61"
		jne tellFormat			; erroneous input
setTo16:
		mov bx,8				; 16 bit bus
		jmp doBusSet
setTo8:
		mov bWidth,"8 " 		; gets screwed up otherwise
		clear bx

doBusSet:
		@tell AttemptMsg
		@tell bWidth
		@tell PromptMsg
		clear ax
		int 16h

		call SetBus
		call GetBus

		@tell NowMsg

OKexit:
		mov al,NUL
exit:
		mov ah,04Ch
		int 21h

main	ENDP


end
